<?php

class ListeGateway
{


    private $con; 

    public function __construct(Connection $con){ 
        $this->con = $con; 
    } 

    public function insert(Liste $l): void{
        $tacheGateway=new TacheGateway($this->con);
        if ($l->getCreator()==null){
            $query = "INSERT INTO Liste VALUES (null, :name, :private, null)";
            $this->con->executeQuery($query, array(':name' => array($l->getName(), PDO::PARAM_STR), ':private' => array($l->getPrivate(), PDO::PARAM_BOOL)));
        }
        else{
            $query = "INSERT INTO Liste VALUES (null, :name, :private, :creator)";
            $this->con->executeQuery($query, array(':name' => array($l->getName(), PDO::PARAM_STR), ':private' => array($l->getPrivate(), PDO::PARAM_BOOL), ':creator' => array($l->getCreator()->getId(), PDO::PARAM_STR)));
        }
        foreach($l->getTaches() as $taches){
            $tacheGateway->insert($taches, $this->getLastId());
        }
    }

    public function delete(Liste $l): void{
        $tacheGateway=new TacheGateway($this->con);
        foreach($l->getTaches() as $taches){
            $tacheGateway->delete($taches);
        }
        $query = "DELETE FROM Liste where id=:id";
        $this->con->executeQuery($query, array(':id' => array($l->getId(), PDO::PARAM_INT)));
    }

    public function update(Liste $l): void{
        $query = "UPDATE Liste SET name=:name, private=:private WHERE id=:id";
        $this->con->executeQuery($query, array(':id' => array($l->getId(), PDO::PARAM_INT), ':name' => array($l->getName(), PDO::PARAM_STR), ':private' => array($l->getPrivate(), PDO::PARAM_BOOL)));
    }

    public function getLastId(): array{
        $query = "SELECT max(id) as oldId FROM Liste";
        $this->con->executeQuery($query, array());
        $results=$this->con->getResults();
        return $results;
    }

    public function findByName(string $name): array{
        $results=[];
        if (!empty($name)){
            $name=$name.'%';
            $query = "SELECT * FROM Liste WHERE name LIKE :name";
            $this->con->executeQuery($query, array(':name' => array($name, PDO::PARAM_STR)));

            $results=$this->con->getResults();
            return $results;
        }
        return $results;
    }

    public function findById(string $id): array{
        $results=[];
        if (!empty($id)){
            $query = "SELECT * FROM Liste WHERE id=:id";
            $this->con->executeQuery($query, array(':id' => array($id, PDO::PARAM_STR)));

            $results=$this->con->getResults();
            return $results;
        }
        return $results;
    }

    public function findByUserId(string $userId): array{
        $query = "SELECT * FROM Liste WHERE idCreator=:idCreator";
        $this->con->executeQuery($query, array(':idCreator' => array($userId, PDO::PARAM_STR)));

        $results=$this->con->getResults();
        return $results;
    }


    public function getPublicList():array{
        $listes = [];
        $taches = array();
        try {

            $query = "SELECT * FROM Liste WHERE private='0'";

            $this->con->executeQuery($query, []);

            $results = $this->con->getResults();
            foreach($results as $row){
                $idListe = $row['id'];
                $queryTaches = "SELECT * FROM Tache WHERE idListe=:idListe";
                $this->con->executeQuery($queryTaches, array(':idListe' => array($idListe, PDO::PARAM_INT)));
                $resultsTaches = $this->con->getResults();
                foreach($resultsTaches as $rowTaches){
                    if($rowTaches['completed']==="0"){
                        $taches[] = new Tache($rowTaches['id'], $rowTaches['name'], $rowTaches['content'],false);
                    }else{
                        $taches[] = new Tache($rowTaches['id'], $rowTaches['name'], $rowTaches['content'],true);
                    }                    
                }
                $listes[] = new Liste($row['id'], $row['name'],false, null, $taches);
                $taches = array();

            }
        }
        catch(PDOException $Exception) {
            echo 'erreur';
            echo $Exception->getMessage();
        }
        return $listes;
    }

    
}

?>